use strict;
use XML::LibXML;

binmode STDOUT, ":utf8";
binmode STDIN, ":utf8";

my $parser = XML::LibXML->new();
$parser->keep_blanks(0);

my $timeLines;
my $targetEntity;

my $entity = $ARGV[1];
my $headfile = $entity;
$headfile=~s/_/ /g;
$headfile=~s/\&/_and_/;
my @targetTokens = split(/_/,$entity);
my $acronym;
foreach my $token (@targetTokens)
{
    $token =~ /(.{1}).*/;
    $acronym .= $1;
}
push @{ $targetEntity }, $entity;
push @{ $targetEntity }, $acronym;

my $track = $ARGV[3];
my $gsEventMentions;
my $listFile;
if ($track eq "B")
{
    $gsEventMentions = &getGSEventMentions($ARGV[4]);
}

$listFile = $ARGV[2]."/".$entity;
if (-e $listFile)
{
    open(F,"<:encoding(UTF-8)", $listFile) or die "can't open $listFile\n";
    my @redirects = <F>;
    close(F);
    foreach my $redirect (@redirects)
    {
	chomp($redirect);
	my @tokens = split(/\_/,$redirect);
	push @{ $targetEntity }, $redirect;
    }
}


open(NORM,"<:encoding(UTF-8)", "pred-normalizations.file");
my $pred_norm;
while(my $line=<NORM>)
{
    chop $line;
    my ($pred,$norm) = split(/ /,$line);
    $pred_norm->{$pred} = $norm;
}
close(NORM);


my $mode_baseline = $ARGV[5];
my $timeLine;
open(FILELIST,$ARGV[0]);
while(my $file=<FILELIST>)
{
    chop $file;
    if ($file=~/\.naf$/)
    {
	my @path = split("/",$file);
	my $file_id=$path[$#path];
	$file_id=~s/^([a-z]+-[0-9]+)[^0-9].+/$1/;
	my $lang = $file_id;
	$lang=~s/^([a-z]+).+/$1/;
	my $doc = $parser->parse_file($file);
	my $naf_root = $doc->documentElement();

	my $tokens;
	foreach my $wf ($naf_root->findnodes('//text/wf'))
	{
	    my $wf_id = $wf->getAttributeNode('id')->textContent;
	    my $wf_sent = $wf->getAttributeNode('sent')->textContent;
	    $wf_sent = $wf_sent - 1;
	    my $wf_word = $wf->textContent;

	    $tokens->{$wf_id}->{word} = $wf_word;
	    $tokens->{$wf_id}->{sent} = $wf_sent;
	}


	my $terms;
	foreach my $term ($naf_root->findnodes('//terms/term'))
	{
	    my $term_id = $term->getAttributeNode('id')->textContent;
	    my $term_lemma = $term->getAttributeNode('lemma')->textContent;
	    my $term_pos = $term->getAttributeNode('morphofeat')->textContent;
	    my $term_word = $term->findnodes('./span/target')->get_node(1)->getAttributeNode('id')->textContent;
	    
	    $terms->{$term_id}->{lemma} = $term_lemma;
	    $terms->{$term_id}->{pos} = $term_pos;
	    $terms->{$term_id}->{word_id} = $term_word;
	    $terms->{$term_id}->{word} = $tokens->{$term_word}->{word};
	    $terms->{$term_id}->{sent} = $tokens->{$term_word}->{sent};
	}

	my @predicates_sorted_list;
	my $predicates;
	my $argh;
	if ($lang eq "en")
	{
	    $argh = "A";
	}
	elsif ($lang eq "es")
	{
	    $argh = "arg";
	}
	foreach my $predicate ($naf_root->findnodes('//srl/predicate'))
	{
	    my $predicate_id = $predicate->getAttributeNode('id')->textContent;
	    my $predicate_target = $predicate->findnodes('./span/target')->get_node(1)->getAttributeNode('id')->textContent;

	    if (!$predicate->exists('./role[@semRole="'.$argh.'M-NEG"]')){
                 if (!$predicate->exists('./role[@semRole="'.$argh.'M-MOD"]') ||
                     ($terms->{$predicate->findnodes('./role[@semRole="'.$argh.'M-MOD"]/span/target[@head="yes"]')->get_node(1)->getAttributeNode('id')->textContent} eq "will")){

			my $predicate_reference = $predicate->findnodes('./externalReferences/externalRef')->get_node(1)->getAttributeNode('reference')->textContent;
			my $predicate_type;
			if ($predicate->exists('./externalReferences/externalRef[@resource="EventType"]'))
			{
			    $predicate_type = $predicate->findnodes('./externalReferences/externalRef[@resource="EventType"]')->get_node(1)->getAttributeNode('reference')->textContent;
			}

			push @predicates_sorted_list, $predicate_id;
			$predicates->{$predicate_id}->{predicate} = $predicate_reference;
			$predicates->{$predicate_id}->{type} = $predicate_type;
			$predicates->{$predicate_id}->{lemma} = $terms->{$predicate_target}->{lemma};
			$predicates->{$predicate_id}->{word} = $terms->{$predicate_target}->{word};
			$predicates->{$predicate_id}->{pos} = $terms->{$predicate_target}->{pos};
			$predicates->{$predicate_id}->{sent} = $terms->{$predicate_target}->{sent};
			foreach my $role ($predicate->findnodes('./role[@semRole="'.$argh.'0" or @semRole="'.$argh.'1"]'))
			{
			    my $role_id = $role->getAttributeNode('id')->textContent;
			    my $role_semRole = $role->getAttributeNode('semRole')->textContent;
			    my $role_span;
			    my $role_head;
			    foreach my $target ($role->findnodes('./span/target'))
			    {
				my $role_target = $target->getAttributeNode('id')->textContent;
				push @{ $role_span }, $role_target;
				if ($target->exists('./@head="yes"'))
				{
				    $role_head = $role_target;
				}
			    }
			    
			    $predicates->{$predicate_id}->{roles}->{$role_semRole}->{span} = $role_span;
			    $predicates->{$predicate_id}->{roles}->{$role_semRole}->{head} = $role_head;
			    $predicates->{$predicate_id}->{roles}->{$role_semRole}->{headl_emma} = $terms->{$role_head}->{lemma};
			    $predicates->{$predicate_id}->{roles}->{$role_semRole}->{head_word} = $terms->{$role_head}->{word};
			}
			if ($predicate->exists('./role[@semRole="'.$argh.'M-TMP"]'))
			{
			    my $tmp = $predicate->findnodes('./role[@semRole="'.$argh.'M-TMP"]')->get_node(1);
			    my $tmp_span;
			    foreach my $target ($tmp->findnodes('./span/target'))
			    {
				my $tmp_target = $target->getAttributeNode('id')->textContent;
				my $tmp_word = $terms->{$tmp_target}->{word_id};
				push @{ $tmp_span }, $tmp_word;
			    }
			    
			    $predicates->{$predicate_id}->{tmp}->{span} = $tmp_span;
			}
		 }
	    }
	}


	my $coreferences;
	my $coref_chains;
	foreach my $coref ($naf_root->findnodes('//coreferences/coref'))
	{
	    my $coref_id = $coref->getAttributeNode('id')->textContent;
	    foreach my $span ($coref->findnodes('./span'))
	    {
		my @span;
		foreach my $target ($span->findnodes('./target'))
		{
		    push @span, $target->getAttributeNode('id')->textContent;
		}
		
		$coreferences->{join(" ",@span)} = $coref_id;
		push @{ $coref_chains->{$coref_id} }, join(" ",@span);
	    }
	}


	my $entities;
	my $entitites_per_sentence;
	foreach my $entity ($naf_root->findnodes('//entities/entity'))
	{
	    my @span;
	    my $sent;
	    foreach my $target ($entity->findnodes('./references/span/target'))
	    {
		my $target_term = $target->getAttributeNode('id')->textContent;
		push @span, $target_term;
		$sent =  $terms->{$target_term}->{sent};
	    }

	    if ($entity->exists('./externalReferences/externalRef'))
	    {
		my $dbpedia = $entity->findnodes('./externalReferences/externalRef')->get_node(1)->getAttributeNode('reference')->textContent;
		$dbpedia=~s/http\:\/\/dbpedia\.org\/resource\///;
		$entities->{join(" ",@span)} = $dbpedia;
		my $new = 1;
		foreach my $dbp (@{ $entitites_per_sentence->{$sent} })
		{
		    if ($dbp eq $dbpedia)
		    {
			$new = 0;
		    }
		}
		if ($new)
		{
		    push @{ $entitites_per_sentence->{$sent} }, $dbpedia;
		}

		if (exists $coreferences->{join(" ",@span)})
		{
		    my $coref_id = $coreferences->{join(" ",@span)};
		    foreach my $mention (@{ $coref_chains->{$coref_id} })
		    {
			$entities->{$mention} = $dbpedia;
			my @mention_terms = split(/ /,$mention);
			foreach my $mention_term (@mention_terms)
			{
			    my $mention_sent =  $terms->{$mention_term}->{sent};
			    my $new = 1;
			    foreach my $dbp (@{ $entitites_per_sentence->{$mention_sent} })
			    {
				if ($dbp eq $dbpedia)
				{
				    $new = 0;
				}
			    }
			    if ($new)
			    {
				push @{ $entitites_per_sentence->{$mention_sent} }, $dbpedia;
			    }
			}
		    }
		}
	    }
	}



	my $timeExpressions;
	foreach my $timex ($naf_root->findnodes('//timeExpressions/timex3[not(starts-with(@value, "P"))]'))
	{
	    my $timex_id = $timex->getAttributeNode('id')->textContent;
	    $timeExpressions->{$timex_id}->{type} = $timex->getAttributeNode('type')->textContent;
	    $timeExpressions->{$timex_id}->{value} = $timex->getAttributeNode('value')->textContent;

	    my $timex_span;
	    foreach my $target ($timex->findnodes('./span/target'))
	    {
		my $timex_target = $target->getAttributeNode('id')->textContent;
		push @{ $timex_span }, $timex_target;
	    }
	    $timeExpressions->{$timex_id}->{span} = $timex_span;
	}


	my $anchorname;
	if ($lang eq "en")
	{
	    $anchorname = "IS_INCLUDED";
	}
	elsif ($lang eq "es")
	{
	    $anchorname = "SIMULTANEOUS";
	}
	my $timeAnchors;
	foreach my $tlink ($naf_root->findnodes('//temporalRelations/tlink[@relType="'.$anchorname.'" and @toType="timex"]'))
	{
	    my $from = $tlink->getAttributeNode('from')->textContent;
	    my $to = $tlink->getAttributeNode('to')->textContent;
	    
	    $timeAnchors->{$from} = $to;
   	}    

	my $dcttmx;
	if ($lang eq "en")
	{
	    $dcttmx = "tmx0";
	}
	elsif ($lang eq "es")
	{
	    $dcttmx = "tmx1";
	}
	my $default_time = $timeExpressions->{$dcttmx }->{value};
	my $default_sent = 0;
	my $default_pos = "";
	my $dct = $timeExpressions->{$dcttmx}->{value};
	my $last_time;
	my $last_sent;
	my $coref_time;
	foreach my $predicate_id (@predicates_sorted_list)
	{
	    my $predicate = $predicates->{$predicate_id}->{predicate};
	    my $predicate_type = $predicates->{$predicate_id}->{type};
	    my $predicate_pos = $predicates->{$predicate_id}->{pos};
	    my $predicate_sent = $predicates->{$predicate_id}->{sent};
	    my $predicate_norm = $pred_norm->{$predicate};
	    my $targeted = 0;
	    foreach my $role_semRole (keys %{ $predicates->{$predicate_id}->{roles} })
	    {
		my $head_lemma = $predicates->{$predicate_id}->{roles}->{$role_semRole}->{head_lemma};
		my $head_word = $predicates->{$predicate_id}->{roles}->{$role_semRole}->{head_word};
		my $entity;
		my @span_terms;
		foreach my $term (@{ $predicates->{$predicate_id}->{roles}->{$role_semRole}->{span} })
		{ 
		    my $lemma = $terms->{$term}->{lemma};
		    push @span_terms, $term;
		}
		if (exists $entities->{join(" ",@span_terms)})
		{
		    $entity = $entities->{join(" ",@span_terms)};
		}
		elsif (exists $entities->{$head_word})
		{
		    $entity = $entities->{$head_word};
		}
		foreach my $targetToken (@{ $targetEntity })
		{
		    if (lc($head_word) eq lc($targetToken) || lc($entity) eq lc($targetToken))
		    {
			$targeted = 1;
		    }
		}
	    }

	    my $isgold = 0;
	    if (exists $gsEventMentions->{$file_id."-".$predicates->{$predicate_id}->{sent}."-".$predicates->{$predicate_id}->{word}})
	    {
	    	$isgold = 1;
	    }
	
	    if (($track eq "A" && $targeted) || ($track eq "B"  && $isgold))
	    {
		if (exists $predicates->{$predicate_id}->{tmp})
		{
		    my $timeValue = &normalizeTMP($timeExpressions, $predicates->{$predicate_id}->{tmp}->{span});
		    if ($timeValue ne "")
		    {
			my ($year,$month,$day) = split(/-/,$timeValue);
			if ($year eq "")
			{
			    $year = "0000";
			}
			if ($month eq "")
			{
			    $month = "00";
			}
			if ($day eq "")
			{
			    $day = "00";
			}

			
			if ($year=~/[0-9]{4}/ && $month=~/[0-9]{2}/ && $day=~/[0-9]{2}/)
			{
			    $timeValue = $year."-".$month."-".$day;
			    push @{ $timeLine->{$year}->{$month}->{$day} }, $file_id."-".$predicates->{$predicate_id}->{sent}."-".$predicates->{$predicate_id}->{word};
			    if ($predicate_norm ne "")
			    {
				$coref_time->{$predicate_norm} = $timeValue;
			    }
			    if ($predicate_pos=~/^V/)
			    {
				$last_time->{$predicate_pos}->{time} =  $timeValue;
				$last_time->{$predicate_pos}->{sent} =  $predicate_sent;
				$default_sent = $predicate_sent;
				$default_pos = $predicate_pos;
				$default_time = $timeValue;
			    }
			}
		    }
		}
		elsif (exists $timeAnchors->{$predicate_id})
		{
		    my $time_id = $timeAnchors->{$predicate_id};
		    my $timeValue = $timeExpressions->{$time_id}->{value};
		    if ($timeValue ne "")
		    {
			my ($year,$month,$day) = split(/-/,$timeValue);
			if ($year eq "")
			{
			    $year = "0000";
			}
			if ($month eq "")
			{
			    $month = "00";
			}
			if ($day eq "")
			{
			    $day = "00";
			}
			
			
			if ($year=~/[0-9]{4}/ && $month=~/[0-9]{2}/ && $day=~/[0-9]{2}/)
			{
			    $timeValue = $year."-".$month."-".$day;
			    push @{ $timeLine->{$year}->{$month}->{$day} }, $file_id."-".$predicates->{$predicate_id}->{sent}."-".$predicates->{$predicate_id}->{word};
			    if ($predicate_norm ne "")
			    {
				$coref_time->{$predicate_norm} = $timeValue;
			    }
			    if ($predicate_pos=~/^V/)
			    {
				$last_time->{$predicate_pos}->{time} =  $timeValue;
				$last_time->{$predicate_pos}->{sent} =  $predicate_sent;			
				$default_sent = $predicate_sent;
				$default_pos = $predicate_pos;
				$default_time = $timeValue;
			    }
			}
		    }
		}
		elsif($mode_baseline eq "dlt")
		{
		    if($predicate_pos=~/^V/ && exists $last_time->{$predicate_pos}->{time})
		    {
			my ($year,$month,$day) = split(/-/,$last_time->{$predicate_pos}->{time});
			if ($predicate_norm ne "")
			{
			    $coref_time->{$predicate_norm} = $last_time->{$predicate_pos}->{time};
			}
			if ($year ne "0000" && $month ne "00" && $day ne "00")
			{
			    push @{ $timeLine->{$year}->{$month}->{$day} }, $file_id."-".$predicates->{$predicate_id}->{sent}."-".$predicates->{$predicate_id}->{word};
			}
			$last_time->{$predicate_pos}->{time} =  $default_time;
			$last_time->{$predicate_pos}->{sent} =  $predicate_sent;
			$default_sent = $predicate_sent;
			$default_pos = $predicate_pos;
		    }
		    elsif($predicate_pos=~/^V/)
		    {
			my ($year,$month,$day) = split(/-/,$dct);
			if ($predicate_norm ne "")
			{
			    $coref_time->{$predicate_norm} = $dct;
			}
			if ($year ne "0000" && $month ne "00" && $day ne "00")
			{
			    push @{ $timeLine->{$year}->{$month}->{$day} }, $file_id."-".$predicates->{$predicate_id}->{sent}."-".$predicates->{$predicate_id}->{word};
			}
			$last_time->{$predicate_pos}->{time} =  $default_time;
			$last_time->{$predicate_pos}->{sent} =  $predicate_sent;
			$default_sent = $predicate_sent;
			$default_pos = $predicate_pos;
		    }
		}
		else
		{
		    my $year = "0000";
		    my $month = "00";
		    my $day = "00";
		    push @{ $timeLine->{$year}->{$month}->{$day} }, $file_id."-".$predicates->{$predicate_id}->{sent}."-".$predicates->{$predicate_id}->{word};
		}
	    }
	}
    }
}
close(FILELIST);


print $headfile."\n";
my $id = 1;
foreach my $year (sort {$a<=>$b} keys %{ $timeLine })
{
    foreach my $month (sort {$a<=>$b} keys %{ $timeLine->{$year} })
    {
	foreach my $day (sort {$a<=>$b} keys %{ $timeLine->{$year}->{$month} })
	{
	    my $one_printed = 0;
	    foreach my $event (@{ $timeLine->{$year}->{$month}->{$day} })
	    {
		my $date = $year."-".$month."-".$day;
		$date=~s/0000/XXXX/;
		$date=~s/-00/-XX/g;
		if ($date ne "XXXX-XX-XX" && $date ne "--")
		{
		    $one_printed = 1;
		    print $id."\t".$date."\t".$event."\n";
		}
	    }
	    if ($one_printed)
	    {
		$id++;
	    }
	}
    }
}

######################################################################

sub getGSEventMentions
{
    my $cat_dir = $_[0];

    my $gsEventMentions;

    opendir(DIR,$cat_dir);
    while(my $file=readdir(DIR))
    {
	if ($file=~/\.txt\.xml/)
	{
	    my $doc = $parser->parse_file($cat_dir."/".$file);
	    my $cat_root = $doc->documentElement();
	    
	    foreach my $D ($cat_root->findnodes('//Document'))
	    {
		my $D_id = $D->getAttributeNode('doc_id')->textContent;
		
		my $tokens;
		foreach my $token ($D->findnodes('./token'))
		{
		    my $token_number = $token->getAttributeNode('number')->textContent;
		    my $token_sentence = $token->getAttributeNode('sentence')->textContent;
		    my $token_id = $token->getAttributeNode('t_id')->textContent;
		    my $token_word = $token->textContent;
		    $tokens->{$token_id} = $D_id."-".$token_sentence."-".$token_word;
		}
		foreach my $event ($D->findnodes('./Markables/EVENT_MENTION/token_anchor'))
		{
		    my $event_anchor_id = $event->getAttributeNode('t_id')->textContent;
		    $gsEventMentions->{$tokens->{$event_anchor_id}} = "Y";
		}
	    }
	}
    }
    closedir(DIR);

    return $gsEventMentions;
}


#######################################################

sub normalizeTMP
{
    my $timeExpressions = $_[0];
    my $span = $_[1];

    my $normalized = "";

    my $max_matches = 0;
    foreach my $timex_id (keys %{ $timeExpressions })
    {
	my $matches = 0;
	foreach my $s1 (@{ $timeExpressions->{$timex_id}->{span} })
	{
	    foreach my $s2 (@{ $span })
	    {
		if ($s1 eq $s2)
		{
		    $matches++;
		}
	    }
	}
	if ($matches > $max_matches)
	{
	    $max_matches = $matches;
	    $normalized = $timeExpressions->{$timex_id}->{value};
	}
    }
    
    return $normalized;
}
