# perl evaluate_selected_events.pl GS_timelines Sys_timelines [debug]

use strict;
use warnings;

use Date::Calc qw(Add_Delta_Days);

my @list_corpus = ("corpus_1_timelines","corpus_2_timelines", "corpus_3_timelines");

my $events0 = 1; #=0 if removed from the evaluation

my $debug = 0;
if(@ARGV > 2){
    $debug = $ARGV[2];
}

my ($globalTP, $globalFN, $globalFP) = (0,0,0);
my ($globalSM, $globalRM) = (0,0);

foreach my $corpus (@list_corpus){

    my @filesRef = glob($ARGV[0].$corpus."/*.txt");
    
    
    if($debug == 1){
	print "\n>>>>>>".$corpus."\n\n";
	print "accuracy\trecall\tprecision\n";
    }

    #TP, FN, FP for one corpus
    my ($totalTP, $totalFN, $totalFP) = (0,0,0);
    my ($totalSM, $totalRM) = (0,0);

    #for each timeline of a corpus
    foreach my $fileR (@filesRef){
	my ($radical) = ($fileR =~ /$ARGV[0]$corpus\/(.*)$/);
	#read the GS timeline
	my %events_ref;
	open(IN,'<'.$fileR);
	while(<IN>){
	    if(/\t/){
		s/\n$//;
		my @elts = split("\t",$_);
		if(($elts[0] ne "0" && $events0 == 0) || $events0 == 1){
		    for(my $i=2;$i<@elts;$i++){
			$events_ref{$elts[$i]} = $elts[1];
		    }
		}
	    }
	}
	close IN;

	#read the system timeline
	if(-e $ARGV[1].$corpus."/".$radical){
	    my %events_sys;
	    open(IN,"<".$ARGV[1].$corpus."/".$radical);
	    while(<IN>){
		if(/\t/){
		    s/\n$//;
		    my @elts = split("\t",$_);
		    if(($elts[0] ne "0" && $events0 == 0) || $events0 == 1){
			for(my $i=2;$i<@elts;$i++){
			    $elts[1] =~ s/^([0-9]{4})\-([1-9]\-)/$1\-0$2/;
			    $elts[1] =~ s/^([0-9]{4}\-[0-9]+)\-([1-9])$/$1\-$2/;
			    $events_sys{$elts[$i]} = $elts[1];
			}
		    }
		}
	    }
	    close IN;

	    #Compute the number of tp, fn and fp for the timeline
	    my ($tp,$fn,$fp) = (0,0,0);
	    my $rm_tmp = 0;
	    my $strict_match = 0;
	    foreach my $ev (keys(%events_ref)){
		if (exists($events_sys{$ev})){
		    if ($events_sys{$ev} eq $events_ref{$ev}){
			$strict_match ++;
		    }
		    #test to compute relaxed match (not used to compute results for the paper)
		    else{
			$events_ref{$ev} =~ s/^([0-9]+-[0-9]+-[0-9]+)(T.*)$/$1/;
			$events_sys{$ev} =~ s/^([0-9]+-[0-9]+-[0-9]+)(T.*)$/$1/;
			

			my @ta_ref = split("-",$events_ref{$ev});
			my @ta_sys = split("-",$events_sys{$ev});

			if($ta_ref[0] eq $ta_sys[0]){
			    $rm_tmp += 0.25;#year
			    if(@ta_ref >= 2 && @ta_sys >= 2){
				if($ta_ref[1] eq $ta_sys[1]){
				    $rm_tmp += 0.25;#month
				    if(@ta_ref == 3 && @ta_sys == 3){
					if($ta_ref[2] eq "XX" || $ta_sys[2] eq "XX"){
					    $rm_tmp += 0.25;
					}
					elsif ($events_ref{$ev} =~ /^[0-9]+-[0-9]+-[0-9]+$/ && $events_sys{$ev} =~ /^[0-9]+-[0-9]+-[0-9]+$/){
					    my ($year1, $month1, $day1) = Add_Delta_Days($ta_ref[0],$ta_ref[1],$ta_ref[2],1);
					    #my ($year2, $month2, $day2) = Add_Delta_Days($ta_ref[0],$ta_ref[1],$ta_ref[2],2);
					    my ($year3, $month3, $day3) = Add_Delta_Days($ta_ref[0],$ta_ref[1],$ta_ref[2],-1);
					    #my ($year4, $month4, $day4) = Add_Delta_Days($ta_ref[0],$ta_ref[1],$ta_ref[2],-2);
					    #if(($ta_sys[0] == $year1 && $ta_sys[1] == $month1 && $ta_sys[2] == $day1) || ($ta_sys[0] == $year2 && $ta_sys[1] == $month2 && $ta_sys[2] == $day2) || ($ta_sys[0] == $year3 && $ta_sys[1] == $month3 && $ta_sys[2] == $day3) || ($ta_sys[0] == $year4 && $ta_sys[1] == $month4 && $ta_sys[2] == $day4)){
					    if(($ta_sys[0] == $year1 && $ta_sys[1] == $month1 && $ta_sys[2] == $day1) || ($ta_sys[0] == $year3 && $ta_sys[1] == $month3 && $ta_sys[2] == $day3)){
						$rm_tmp += 0.25;
					    }
					}
				    }
				}
			    }
			}
			
		    }
		    $tp++;
		}
	    }
	    
	    $totalTP += $tp;

	    $totalSM += $strict_match;
	    $totalRM += $rm_tmp;
	}
    }

    #compute micro-average recall, precision and fscore for one corpus

    $globalTP += $totalTP;


    $globalSM += $totalSM;
    $globalRM += $totalRM;

    print "\n======== ".$corpus." ==========\n";


    #my $time_anchor_acc = ($totalSM+$totalRM)/$totalTP;
    my $time_anchor_acc = $totalSM/($totalTP);
    my $time_anchor_recall = $totalSM/($totalTP+$totalFN);
    my $time_anchor_precision = $totalSM/($totalTP+$totalFP);
    my $time_anchor_fscore = 0;
    if ($time_anchor_recall != 0 || $time_anchor_precision != 0){
	$time_anchor_fscore = ($time_anchor_recall*$time_anchor_precision*2)/($time_anchor_recall+$time_anchor_precision);
    }

    print $time_anchor_recall."\t".$time_anchor_precision."\t".$time_anchor_fscore."\n";
    print "nb events: ".$totalTP."\n";
    print "time anchors: \t".$time_anchor_acc."\n";
}


#compute micro-average recall, precision and fscore for the 3 corpora

print "\n======== TOTAL ==========\n";

my $global_time_anchor_acc = $globalSM/$globalTP;
print "time anchors: \t".$global_time_anchor_acc."\n";
