# perl evaluate_selected_events.pl GS_timelines Sys_timelines [debug]

use strict;
use warnings;

my @list_corpus = ("corpus_1_timelines","corpus_2_timelines", "corpus_3_timelines");

my $debug = 0;
if(@ARGV > 2){
    $debug = $ARGV[2];
}

my ($globalTP, $globalFN, $globalFP) = (0,0,0);

foreach my $corpus (@list_corpus){

    my @filesRef = glob($ARGV[0].$corpus."/*.txt");
    
    
    if($debug == 1){
	print "\n>>>>>>".$corpus."\n\n";
	print "accuracy\trecall\tprecision\n";
    }

    #TP, FN, FP for one corpus
    my ($totalTP, $totalFN, $totalFP) = (0,0,0);

    #for each timeline of a corpus
    foreach my $fileR (@filesRef){
	my ($radical) = ($fileR =~ /$ARGV[0]$corpus\/(.*)$/);

	#read the GS timeline
	my %events_ref;
	open(IN,'<'.$fileR);
	while(<IN>){
	    if(/\t/){
		s/\n$//;
		my @elts = split("\t",$_);
		for(my $i=2;$i<@elts;$i++){
		    $events_ref{$elts[$i]} = "";
		}
	    }
	}
	close IN;

	#read the system timeline
	if(-e $ARGV[1].$corpus."/".$radical){
	    my %events_sys;
	    open(IN,"<".$ARGV[1].$corpus."/".$radical);
	    while(<IN>){
		if(/\t/){
		    s/\n$//;
		    my @elts = split("\t",$_);
		    for(my $i=2;$i<@elts;$i++){
			$events_sys{$elts[$i]} = "";
		    }
		}
	    }
	    close IN;

	    #Compute the number of tp, fn and fp for the timeline
	    my ($tp,$fn,$fp) = (0,0,0);
	    foreach my $ev (keys(%events_ref)){
		if (exists($events_sys{$ev})){
		    $tp++;
		}
		else{
		    $fn++;
		}
	    }
	    foreach my $ev (keys(%events_sys)){
		if(not(exists($events_ref{$ev}))){
		    $fp++;
		}
	    }
	    
	    my $accurracy = 0;
	    if ($tp != 0 || $fn != 0){
		$accurracy = $tp/($tp+$fn);
	    }
	    
	    
	    
	    $totalTP += $tp;
	    $totalFN += $fn;
	    $totalFP += $fp;
	    
	    my ($recall, $precision) = (0,0);
	    
	    if($tp != 0 || $fn != 0){
		$recall = $tp/($tp+$fn);
	    }
	    if($tp != 0 || $fp != 0){
		$precision = $tp/($tp+$fp);
	    }
	    
	    if($debug == 1){
		print $radical."\n";
		print $accurracy."\t".$recall."\t".$precision."\n";
	    }
	}
    }

    #compute micro-average recall, precision and fscore for one corpus

    my ($totalR, $totalP, $totalF) = (0,0,0);

    if($totalTP != 0 || $totalFN != 0){
	$totalR = $totalTP/($totalTP+$totalFN);
    }
    
    if($totalTP != 0 || $totalFP != 0){
	$totalP = $totalTP/($totalTP+$totalFP);
    }
    
    if($totalP != 0 || $totalR != 0){
	$totalF = ($totalP*$totalR*2)/($totalP+$totalR);
    }    

    $globalTP += $totalTP;
    $globalFN += $totalFN;
    $globalFP += $totalFP;


    print "\n======== ".$corpus." ==========\n";
    print "recall\tprecision\tfscore\n";
    print $totalR."\t".$totalP."\t".$totalF."\n";
}


#compute micro-average recall, precision and fscore for the 3 corpora

my ($globalR, $globalP, $globalF) = (0,0,0);
if($globalTP != 0 || $globalFN != 0){
    $globalR = $globalTP/($globalTP+$globalFN);
}
if($globalTP != 0 || $globalFP != 0){
    $globalP = $globalTP/($globalTP+$globalFP);
}

if($globalR != 0 || $globalP != 0){
    $globalF = ($globalR*$globalP*2)/($globalR+$globalP);
}

print "\n======== TOTAL ==========\n";
print "recall\tprecision\tfscore\n";
print $globalR."\t".$globalP."\t".$globalF."\n";
